// USBtemp.cpp :	Read temperature in a loop and write results to file in argument
//					USBtemp [-o filename] [-s serialnumber] [-d] [-b] [-h] [-i timeinterval]
//					-o filename: write to this file
//					-o+ filename: dito, but appends to the end of file
//					-s serialnumber: get data from this device
//					-i interval between two samples in seconds
//					-b used in a batch, get a single value and exit (if -i is undefined)
//					   or overwrite the outfile if -i is defined
//					-d print out debug statements
//					-h help 
// (C) Copyright 2004-2010 Cleware GmbH, Germany
//
// 07.07.2009	3.2		added -o+ append modus
// 08.07.2009	3.3		when -b is used together with -i, dont exit
// 13.01.2011	3.4		-h option filled with life
// 08.03.2011	3.5		-2  use temperature function FCW2GetTemperature

#ifdef CCLINUX
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#endif	// CCLINUX

#include "stdafx.h"
#include "USBaccess.h"

#ifdef CCLINUX
#define Sleep(ms) usleep( (ms) * 1000 )
#endif	// CCLINUX

const float USBtempVersion = (float) 3.5 ;

int 
main(int argc, char* argv[]) {
	CUSBaccess CWusb ;

	char *filename = 0 ;
	int doDebug = 0 ;
	int showHelp = 0 ;
	char *progname = *argv ;
	int serialnumber = -1 ;
	int devID = -1 ;
	int batch=0 ;
	int interval = -1111 ;
	FILE *out ;
	char *fileMode = "w+" ;
	int useTemp2function = 0 ;

	for (argc--, argv++ ; argc > 0 ; argc--, argv++) {
		if (**argv == '-' 
#ifndef CCLINUX
							|| **argv == '/'
#endif
							) {
			switch (argv[0][1]) {
				case '2':
					useTemp2function = 1 ;
					break ;
				case 'O':
				case 'o':
					if (argc == 1) {
						printf("missing filename\n") ;
						break ;
						}
					if (argv[0][2] == '+')
						fileMode = "a+" ;
					filename = *++argv ;
					argc-- ;
					break ;
				case 'S':
				case 's':
					if (argc == 1) {
						printf("missing serial number\n") ;
						break ;
						}
					argv++ ;
					argc-- ;
					if (sscanf(*argv, "%d", &serialnumber) != 1)
						printf("serial number illegal (%s)\n", *argv) ;
					break ;
				case 'I':
				case 'i':
					if (argc == 1) {
						printf("missing interval\n") ;
						break ;
						}
					argv++ ;
					argc-- ;
					if (sscanf(*argv, "%d", &interval) != 1)
						printf("interval illegal (%s)\n", *argv) ;
					else
						interval *= 1000 ;		// convert to ms
					break ;
				case 'D':
				case 'd':
					doDebug = 1 ;
					break ;
				case 'B':
				case 'b':
					batch = 1 ;
					break ;
				case 'H':
				case 'h':
					showHelp = 1 ;
					break ;
				case '?':
					showHelp = 1 ;
					break ;
				default:
					printf("unknown argument %s\n", *argv) ;
					showHelp = 1 ;
					break ;
				}
			}
		}

	if (showHelp) {
		printf("USBtemp [-o filename] [-s serialnumber] [-d] [-b] [-h] [-i timeinterval]\n") ;
		printf("-o filename: write data to this file\n") ;
		printf("-o+ filename: dito, but appends to the end of file\n") ;
		printf("-s serialnumber: get data from this device\n") ;
		printf("-i interval between two samples in seconds\n") ;
		printf("-b used in a batch, get a single value and exit (if -i is undefined)\n") ;
		printf("   or overwrite the outfile if -i is defined\n") ;
		printf("-d print out debug statements\n") ;
		printf("-h print this \n") ;
		printf("(C) Copyright 2004-2010 Cleware GmbH, Germany\n") ;
		return 0 ;
		}

	int USBcount = CWusb.OpenCleware() ;
	if (doDebug)
		printf("USBtemp V %2.2f found %d devices, using USBaccess version %d\n", 
				USBtempVersion, USBcount, CWusb.GetDLLVersion()) ;

	for (devID=0 ; devID < USBcount ; devID++) {
		int devType = CWusb.GetUSBType(devID) ;
		if (		devType == CUSBaccess::TEMPERATURE_DEVICE 
				 || devType == CUSBaccess::TEMPERATURE2_DEVICE
				 || devType == CUSBaccess::ADC0800_DEVICE
				 || devType == CUSBaccess::HUMIDITY1_DEVICE
				 || devType == CUSBaccess::HUMIDITY2_DEVICE) {
			if (serialnumber < 0)
				break ;
			if (serialnumber == CWusb.GetSerialNumber(devID))
				break ;
			}
		}
	if (devID < 0 || devID >= USBcount) {
		printf("no valid device found\n") ;
		return 0 ;
		}

	if (batch && interval > 0)
		batch = 2 ;
	if (interval <= 0)
		interval= 1200 ;
	if (filename != 0) {
		out = fopen(filename, fileMode) ;
		if (out == 0) {
			printf("failed to open %s\n", filename) ;
			return 0 ;
			}
		}
	else 
		out = stdout ;

	int devType = CWusb.GetUSBType(devID) ;
	if (		devType == CUSBaccess::TEMPERATURE_DEVICE 
			 || devType == CUSBaccess::TEMPERATURE2_DEVICE) {
		if (!useTemp2function) {		// direct temp call should handle this themself
			CWusb.ResetDevice(devID) ;
			Sleep(500) ;		// wait a bit to settle after reset
			}

		while (1) {			// forever
#ifndef CCLINUX
			time_t now ;
			time(&now) ;
			CTime Now = now ;
			CString s = Now.Format("%d.%m.%y %H:%M:%S") ;
#endif	// CCLINUX
			double temperatur ;
			if (useTemp2function) {
				temperatur = CWusb.GetTemperature(devID) ;
				}
			else {
				int	   zeit ;
				int retry ;
				for (retry = 20 ; retry > 0 ; retry--) {
					if (!CWusb.GetTemperature(devID, &temperatur, &zeit)) {
						if (doDebug)
							printf("GetTemperature(%d) failed\n", devID) ;
						Sleep(200) ;		// wait a bit to settle after reset
						}
					else
						break ;
					}

				if (retry == 0) {
					if (doDebug)
						printf("Get temparature failed - do reset an try again\n") ;
					CWusb.ResetDevice(devID) ;
					Sleep(500) ;		// wait a bit to settle after reset
					continue ;
					}
				}
#ifdef CCLINUX
			printf("%.3lfC\n", temperatur) ;
#else	// CCLINUX
			CString t ;
			t.Format("  -  %.3lfC\n", temperatur) ;
			s += t ;
			const char *str = LPCTSTR(s) ;
			fputs(str, out) ;
			fflush(out) ;
#endif	// CCLINUX
			if (batch == 1)
				break ;
			Sleep(interval) ;
			if (batch == 2) {	// close path and reopen
				if (filename != 0) {
					fclose(out) ;
					out = fopen(filename, fileMode) ;
					if (out == 0) {
						printf("failed to open %s\n", filename) ;
						return 0 ;
						}
					}
				}
			}
		}

	if (devType == CUSBaccess::HUMIDITY1_DEVICE || devType == CUSBaccess::HUMIDITY2_DEVICE) {
		if (!useTemp2function && !batch) {		// in batch, the device is typically active
			CWusb.ResetDevice(devID) ;
			Sleep(500) ;		// wait a bit to settle after reset

			CWusb.StartDevice(devID) ;
			Sleep(1200) ;		// wait for the first values
			}

		int startFailed=0 ;
		while (1) {		// forever
			double temperatur=-300., humidity=0. ;
			double temperatur1=-300., humidity1=0. ;
			double temperatur2=-300., humidity2=0. ;
			int	   zeit ;
#ifndef CCLINUX
			time_t now ;
			time(&now) ;
			CTime Now = now ;
			CString s = Now.Format("%d.%m.%y %H:%M:%S") ;
#endif	// CCLINUX

			int getOk = 0 ;
			if (useTemp2function) {
				temperatur = CWusb.GetTemperature(devID) ;
				humidity = CWusb.GetHumidity(devID) ;
				}
			else {
				int retry ;
				for (retry=10 ; retry > 0 && !getOk ; retry--) {
					getOk = CWusb.GetTemperature(devID, &temperatur, &zeit) ;
					if (!getOk)
						Sleep(100) ;	// retry after 100 ms
					}


				if (!getOk) {
					if (doDebug)
						printf("GetTemperature(%d) failed\n", devID) ;
					}
				else {
					for (getOk=0, retry=10 ; retry > 0 && !getOk ; retry--) {
						getOk = CWusb.GetHumidity(devID, &humidity, &zeit) ;
						if (!getOk)
							Sleep(100) ;	// retry after 100 ms
						}
					if (!getOk && doDebug)
						printf("GetHumidity(%d) failed\n", devID) ;
					}
				if (devType == CUSBaccess::HUMIDITY2_DEVICE) {		// this is the double sensor, try to get both sub values
					int failed = 0 ;
					for (getOk=0, retry=10 ; retry > 0 && !getOk ; retry--) {
						getOk = CWusb.GetTemperature(devID, &temperatur1, &zeit, 1) ;
						if (!getOk)
							Sleep(100) ;	// retry after 100 ms
						}
					if (!getOk)
						failed++ ;
					for (getOk=0, retry=10 ; retry > 0 && !getOk ; retry--) {
						getOk = CWusb.GetHumidity(devID, &humidity1, &zeit, 1) ;
						if (!getOk)
							Sleep(100) ;	// retry after 100 ms
						}
					if (!getOk)
						failed++ ;
					for (getOk=0, retry=10 ; retry > 0 && !getOk ; retry--) {
						getOk = CWusb.GetTemperature(devID, &temperatur2, &zeit, 2) ;
						if (!getOk)
							Sleep(100) ;	// retry after 100 ms
						}
					if (!getOk)
						failed++ ;
					for (getOk=0, retry=10 ; retry > 0 && !getOk ; retry--) {
						getOk = CWusb.GetHumidity(devID, &humidity2, &zeit, 2) ;
						if (!getOk)
							Sleep(100) ;	// retry after 100 ms
						}
					if (!getOk)
						failed++ ;
					if (failed) {
						if (doDebug)
							printf("Get Subdevices(%d) %d calls failed\n", devID, failed) ;
						getOk = 1 ;		// force print 
						}
					}

				if (!getOk) {
					if (startFailed > 2) {
						CWusb.ResetDevice(devID) ;
						Sleep(500) ;		// wait a bit to settle after reset
						}
					CWusb.StartDevice(devID) ;	// restart device
					Sleep(1200) ;
					startFailed++ ;
					continue ;
					}
				}
			startFailed = 0 ;
			if (devType == CUSBaccess::HUMIDITY2_DEVICE) {
#ifdef CCLINUX
				printf("%.2lf dC / %.2lf dC = %.2lf dC, %.2lf / %.2lf = %.2lf %% RH\n", 
					temperatur1, temperatur2, temperatur, 
					humidity1, humidity2, humidity) ;
#else	// CCLINUX
				CString t ;
				t.Format("  -  %.2lf dC / %.2lf dC = %.2lf dC, %.2lf / %.2lf = %.2lf %% RH\n",
					temperatur1, temperatur2, temperatur, 
					humidity1, humidity2, humidity) ;
				s += t ;
				const char *str = LPCTSTR(s) ;
				fputs(str, out) ;
				fflush(out) ;
#endif	// CCLINUX
				}
			else {
#ifdef CCLINUX
				printf("%.2lfC, %.2lf %% RH\n", temperatur, humidity) ;
#else	// CCLINUX
				CString t ;
				t.Format("  -  %.2lf dC, %.2lf %% RH\n", temperatur, humidity) ;
				s += t ;
				const char *str = LPCTSTR(s) ;
				fputs(str, out) ;
				fflush(out) ;
#endif	// CCLINUX
				}
			if (batch == 1)
				break ;
			Sleep(interval) ;
			if (batch == 2) {	// close path and reopen
				if (filename != 0) {
					fclose(out) ;
					out = fopen(filename, fileMode) ;
					if (out == 0) {
						printf("failed to open %s\n", filename) ;
						return 0 ;
						}
					}
				}
			}
		}

	if (devType == CUSBaccess::ADC0800_DEVICE) {
		if (!batch) {		// in batch, the device is typically active
			CWusb.ResetDevice(devID) ;
			Sleep(500) ;		// wait a bit to settle after reset

			CWusb.StartDevice(devID) ;
			Sleep(1200) ;		// wait for the first values
			}

		int startFailed=0 ;
		while (1) {		// forever
			int getOk = 0 ;
			double temperatur=-300., humidity=0. ;
#ifndef CCLINUX
			time_t now ;
			time(&now) ;
			CTime Now = now ;
			CString s = Now.Format("%d.%m.%y %H:%M:%S") ;
#endif	// CCLINUX

				const int maxBufSize = 8 ;
				int bufSize = 6 ;
				unsigned char buf[maxBufSize] ;
				static int sequenceNumber = 1 ;
				buf[0] = CUSBaccess::GetInfo ;
				static int sensorNumber = 0 ;
/*				if (sensorNumber)
					sensorNumber = 0 ;
				else
					sensorNumber = 1 ;
toggle channels later */
				buf[1] = sensorNumber ;
				buf[2] = sequenceNumber ;
				CWusb.SetValue(devID, buf, 3) ;
				Sleep(25) ;
				for (int timeout=50 ; timeout > 0 ; timeout--) {
					if (CWusb.GetValue(devID, buf, 4)) {
						if ((buf[0] & 0x80) == 0) {	// valid bit
							Sleep(25) ;
							continue ;
							}
						if (buf[1] != sequenceNumber)
							continue ;				// no sleep, just kill the fifo
						getOk = 1 ;
						break ;
						}
					}
				CString t ;
				if (getOk) {
					int adcVal = (buf[2] << 8) + buf[3] ;
					static int sensorNumber = 0 ;
					double voltage = (adcVal * 5.181) / 0xfff ;
					double prommVoltage = (adcVal * 34.) / 0xfff ;
					t.Format(" usb returns  Channel %d: %lf V\n", 
												buf[0] & 0x0f, voltage) ;
					}
				else
					t.Format(" usb.GetValue failed (0x%02x 0x%02x 0x%02x 0x%02x), seq=%d\n", 
													buf[0], buf[1], buf[2], buf[3], sequenceNumber) ;
				if (++sequenceNumber >= 128)
					sequenceNumber = 1 ;
#ifdef CCLINUX
//				printf("%.2lfC, %.2lf %% RH\n", temperatur, humidity) ;
#else	// CCLINUX
				s += t ;
				const char *str = LPCTSTR(s) ;
				fputs(str, out) ;
				fflush(out) ;
#endif	// CCLINUX
			if (batch == 1)
				break ;
			Sleep(interval) ;
			if (batch == 2) {	// close path and reopen
				if (filename != 0) {
					fclose(out) ;
					out = fopen(filename, fileMode) ;
					if (out == 0) {
						printf("failed to open %s\n", filename) ;
						return 0 ;
						}
					}
				}
			}
		}

		CWusb.CloseCleware() ;

	return 0;
	}

